# -*- coding: iso-8859-1 -*-
'''
main.py -- actual implementation of all macros in the package.

= Attention Boxes for MoinMoin =

In Wikipedia, it is an established practice to flag pages that need some kind of 
attention using visually styled "message boxes" located somewhere on the page. 
(An example of "some kind of need attention" is a page that needs cleanup.)

This is (the beginning of) an endeavor to employ Attention Boxes in MoinMoin.

The code has been tested with MoinMoin 1.3.4, but is should work with later versions as well 
(due to the fact that the code does not rely on moin code at all).

To list pages needing attention, use:

    [[FullSearch(text:[[Attention()]]

To use the visual styles (CSS and images), read {{{wiki\htdocs\_css\.readme.txt}}}.

I suggest creating a PagesNeedingAttention with the following content:
----
#format wiki
#language en
#pragma section-numbers off

== Pages Needing Attention ==

[[FullSearch(text:regex:\[\[Needs(Review|Cleanup|Rewrite)(\(.*\))?\]\])]]
##[[FullSearch(text:[[Needs)]]

== Examples Of Attention Boxes ==

[[InfoBox(This is an informative message)]]
[[Attention(NeedsReview::Please check consistency and completeness!)]]
[[Attention(NeedsCleanup::This is a topic mix => Split into distinct articles and interlink!)]]
[[Attention(NeedsRewrite::This page is a mess!)]]
----
[[Attention(GreenBox::I'm green)]]
[[Attention(RedBox::I'm red)]]
[[Attention(BlueBox::I'm blue)]]
[[Attention(YellowBox::I'm yellow)]]
[[Attention(ColoredBox::firebrick::I'm __any__ color!)]]
----

Cheers and happy attention drawing

Zoran Isailovski

----
Copyright (C) 2007 Zoran Isailovski
Licensed under the Apache License, Version 2.0 (see http://www.apache.org/licenses/LICENSE-2.0)
'''

DEBUG = True

######################################################################
## 
## Moin Interface
## 
######################################################################

Dependencies = ['namespace'] # footnote: should not be cached

def execute(Run, macro, args):
    def FormatArgsAndRun(args, macro, Run=Run):
        args = format(args, macro.request)
        return Run(args, macro)
    return do(FormatArgsAndRun, macro, args)

def do(Process, macro, args):
    try:
        return Process(args or '', macro)
    except:
        import sys, traceback
        output = traceback.format_exception( *sys.exc_info() )
        if not DEBUG: output = output[-1:]
        output = ''.join( output )
        return ERROR_FORMAT % (Process.__name__, args, output)

ERROR_FORMAT = '''\
<div class="traceback" style="background-color:red; color:lightyellow; font-weight: bold">
<center>%s(%s)</center>
<pre class="traceback" style="background-color:yellow; color:red; font: 11px verdana; border: solid red">\
%s\
</pre>
</div>
'''

######################################################################
## 
## Auxiliaries
## 
######################################################################
import re
from cStringIO import StringIO
from MoinMoin import config
from MoinMoin.parser.wiki import Parser

RX_ARGS_PFX_MSG = re.compile(r'(?:\s*(.*?)\s*::)?\s*(.*)\s*')
RX_WHITESPACE = re.compile(r'\s+')

def format(text, request):
    ##text = request.formatter.text(text)
    text = text.replace(r'\n', '\n\n')
    text = formatAsWiki(text, request)
    return text

def formatAsWiki(text, request):
    buf = StringIO()
    request.redirect(buf)
    try:
        ##request.form = request.args = request.setup_args({})
        parser = Parser(text, RequestWrapper(request))
        parser.format(request.formatter)
        return unicode(buf.getvalue(), config.charset)
    finally:
        request.redirect()

class RequestWrapper:
    def __init__(self, request):
        self.__request = request
    def __getattr__(self, key):
        return getattr(self.__request, key)
    def write(self, s):
        if isinstance(s, unicode):
            s = s.encode(config.charset)
        self.__request.write(s)

def SplitPrefixAndMessage(s):
    m = RX_ARGS_PFX_MSG.match(s)
    if m:
        prefix, msg = m.groups()
    else:
        prefix, msg = None, s
    return prefix, msg

######################################################################
## 
## Generic Attention Box
## 
######################################################################

def Attention(s, macro):
    kind, msg = SplitPrefixAndMessage(s)
    if not kind: kind = 'InfoBox'
    try:
        f = eval(kind)
    except (SyntaxError, NameError):
        f = None
    assert _IsValidMacroFunc(f), _AttentionErrorMsg(kind)
    msg = format(msg, macro.request)
    return f(msg, macro)

def _AttentionErrorMsg(kind):
    return "'%s' is not a valid attention type" % kind

def _IsValidMacroFunc(f):
    return (
        callable(f)
        and f is not Attention
        and not f.__name__.startswith('_')
        and hasattr(f, 'func_code')
        and f.func_code.co_argcount == 2
        and f.func_code.co_varnames[:2] == ('s', 'macro')
        )

######################################################################
## 
## Specific Attention Boxes
## 
######################################################################

def InfoBox(s, macro):
    return InfoBox_FORMAT % s

InfoBox_FORMAT = '''
<div class="messagebox InfoBox">
  <div class="message">
    %s
  </div>
</div>
'''

def NeedsReview(s, macro):
    return NeedsReview_FORMAT % s

NeedsReview_FORMAT = '''
<div class="messagebox NeedsReview">
  <h3>A <em><big>review</big></em> has been requested for this page</h3>
  <div class="message">
    %s
  </div>
</div>
'''

def NeedsCleanup(s, macro):
    return NeedsCleanup_FORMAT % s

NeedsCleanup_FORMAT = '''
<div class="messagebox NeedsCleanup">
  <h3>A <em><big>cleanup</big></em> has been sugested for this page</h3>
  <div class="message">
    %s
  </div>
</div>
'''

def NeedsRewrite(s, macro):
    return NeedsRewrite_FORMAT % s

NeedsRewrite_FORMAT = '''
<div class="messagebox NeedsRewrite">
  <h3>A <em><big>rewrite</big></em> has been sugested for this page</h3>
  <div class="message">
    %s
  </div>
</div>
'''

######################################################################
## 
## Colored Boxes
## 
######################################################################

def GreenBox(s, macro):
    color = '#80FF80'
    return ColoredBox_FORMAT % (color, s)

def RedBox(s, macro):
    color = '#FF4000'
    return ColoredBox_FORMAT % (color, s)

def BlueBox(s, macro):
    color = '#80E0FF'
    return ColoredBox_FORMAT % (color, s)

def YellowBox(s, macro):
    color = '#FFFFA0'
    return ColoredBox_FORMAT % (color, s)

ColoredBox_FORMAT = '''
<div class="messagebox ColoredBox" style="background-color:%s">
  <div class="message">
    %s
  </div>
</div>
'''

def ColoredBox(s, macro):
    color, msg = SplitPrefixAndMessage(s)
    return ShowColoredBox(color, msg, macro)

ColoredBox_FORMAT0 = '''
<div class="messagebox ColoredBox">
  <div class="message">
    %s
  </div>
</div>
'''

def ShowColoredBox(color, msg, macro):
    msg = format(msg, macro.request)
    if color and not RX_WHITESPACE.search(color):
        return ColoredBox_FORMAT % (color, msg)
    else:
        ##return InfoBox_FORMAT % msg
        return ColoredBox_FORMAT0 % msg
